<?php
/*
Plugin Name: Draft Notify
Plugin URI: http://www.touchoftechnology.com/draft-notify-wp-plugin-to-send-an-email-when-an-author-saves-a-draft/
Description: When an author saves a post, it sends out an email based on the settings.  Either to a single email address, or to all users above a certain access level.  You can also select whether or not to send emails for revisions.
Author: Andrew Hallman
Version: 1.5
Author URI: http://www.touchoftechnology.com
*/


/*
* The save function
*/

function dddn_process($id) {
    
        // check for spam first, if checked.
        if(function_exists("spc_check_post"))
        {
            $savedLinkSpam = get_option('draftnotifyemaillinkspam', false);
            if($savedLinkSpam)
            {
                spc_check_post($id, false);
            }
        }
    
	global $wpdb;
	$prefix = $wpdb->prefix;
	// emails anyone on or above this level, unless a specific email address is entered.
	$accessLevel = get_option('draftnotifyuserlevel');  
        $authorLevel = get_option('draftnotifyauthorlevel', 0);
	$emailAddress = get_option('draftnotifyemail');
        $emailForRevisions = get_option('draftnotifyemailrevisions');
	
        $sendForPages = (get_option('draftnotifyposttypes1')!==false && get_option('draftnotifyposttypes1')=="page")?true:false;
        $sendForPosts = (get_option('draftnotifyposttypes2')!==false && get_option('draftnotifyposttypes2')=="post")?true:false;
        
                    // Author's first and last name are found in the metadata table.
	$result = $wpdb->get_row("
		SELECT 	{$prefix}posts.ID, {$prefix}posts.post_status, {$prefix}posts.post_title, {$prefix}posts.post_modified, {$prefix}posts.post_type,
                        user_login, display_name, user_email, 
                        {$prefix}usermeta.meta_value as fname, 
                        meta2.meta_value as lname, meta3.meta_value as access_level
		FROM 	{$prefix}posts, {$prefix}users
                            LEFT JOIN ({$prefix}usermeta) ON
                                    ({$prefix}usermeta.user_id = {$prefix}users.ID AND 
                                     {$prefix}usermeta.meta_key = 'first_name' )
                            LEFT JOIN ({$prefix}usermeta as meta2) ON
                                    (meta2.user_id = {$prefix}users.ID AND 
                                     meta2.meta_key = 'last_name' )
                            LEFT JOIN ({$prefix}usermeta as meta3) ON
                                    (meta3.user_id = {$prefix}users.ID AND
                                    meta3.meta_key = 'wp_user_level' )
		WHERE 	{$prefix}posts.post_author = {$prefix}users.ID AND {$prefix}posts.ID = '$id' ");

                // last line is the checkboxes for post type in the settings page.
	if ( (($result->post_status == "draft") || ($result->post_status == "pending")) && 
                $result->post_parent == 0  && 
                (intval($authorLevel) <= 0 || $result->access_level<=intval($authorLevel)) &&
               ( ($sendForPosts && $result->post_type=="post") || ($sendForPages && $result->post_type=="page")) ) {

            // first check if there are revisions, or if this is the first save.
            $canEmail = true;
            if(!$emailForRevisions)
            {
                // only check if the setting says to ignore revisions.
                
                $revisionCheck = $wpdb->get_results("
                    SELECT 	* 
                    FROM 	{$prefix}posts
                    WHERE 	{$prefix}posts.post_parent = '$id' AND 
                            {$prefix}posts.post_name != '{$id}-autosave%' ");
                if(count($revisionCheck) > 0)
                {
                    $canEmail = false;
                }
                
            }
            
            // also do the manual check for the autosave of the first draft.  
            $alreadyEmailedList = get_option('draftnotifyemailedlist');
            $emailedEx = explode("|", $alreadyEmailedList);
            
            if(in_array($id, $emailedEx))
            {
                $canEmail = false;
            }
            
            if($canEmail)
            {
                
                $message = get_option('draftnotifycustomheader');
		$message .= "A draft was created: '" . get_bloginfo('name') . "'\n<br>";
		$message .= "Post Title: " . $result->post_title . "\n<br>";
		$message .= "Post Modified: " . $result->post_modified . "\n<br>";
		$message .= "Author's Username: " . $result->user_login . "\n<br>";
		
                
                $savedEmailPostLinkVal = get_option('draftnotifyemailpostlink');
                $savedEmailAuthorNameVal = get_option('draftnotifyemailauthorname');
                $savedEmailAuthorEmailAddressVal = get_option('draftnotifyemailauthoremail');
                
                if($savedEmailAuthorNameVal)
                {
                    $message .= "Author's Real Name: {$result->fname} {$result->lname }\n<br>";
                }
                
                
                if($savedEmailAuthorEmailAddressVal)
                {
                    $message .= "Author's Email Address: " . $result->user_email . "\n<br>";
                }
                
                if($savedEmailPostLinkVal)
                {
                    $message .= "\n<br>Link to Pending Posts: <a href='" . home_url('/') . "wp-admin/edit.php?post_status=pending&post_type=post' target='_blank'>here</a>\n<br>";
                    $message .= "\n<br>Edit this post: <a href='" . home_url('/') . "wp-admin/edit.php?post=" . $result->ID . "&action=edit' target='_blank'>here</a>\n<br>";
                }
                

                $message .= get_option('draftnotifycustomfooter');
		
                $emailEx = explode(",", $emailAddress);
		$recipient = $emailEx[0];
		if($recipient == "")
		{
			$users = $wpdb->get_results("SELECT user_id FROM {$prefix}usermeta 
			WHERE 	{$prefix}usermeta.meta_key = 'wp_user_level' AND 
					{$prefix}usermeta.meta_value >= " . $accessLevel);
		
			foreach ($users as $user) 
                        {			
				$user_info = get_userdata($user->user_id);
				if($recipient != ""){$recipient .= ',';}
				$recipient .= $user_info->user_email; 
                        }
		} 
		
                $headers  = 'MIME-Version: 1.0' . "\r\n";
                $headers .= 'Content-type: text/html; charset=iso-8859-1' . "\r\n";
                
                if(get_option('draftnotifyemailfrom')!==false && get_option('draftnotifyemailfrom')!=""){
                $headers .= 'From: ' . get_option('draftnotifyemailfrom') . "\r\n";}
                
                if(get_option('draftnotifyemailreply')!==false && get_option('draftnotifyemailreply')!=""){
                $headers .= 'Reply-To: ' . get_option('draftnotifyemailreply') . "\r\n";}
                
                $cc = "";
                $usedEmails = array($recipient);  // so duplicates aren't sent.
                for($x=0;$x<20;$x++)
                {
                    $tempEmail = get_option("draftnotifyadminemail{$x}");
                    if($tempEmail !== false && $tempEmail != "" && !in_array($tempEmail, $usedEmails) )
                    {
                        array_push($usedEmails, $tempEmail);
                        if($cc!=""){$cc.=",";}
                        $cc.=$tempEmail;
                    }
                }
                
                // check for multiple emails in the address field.  // skip the first one.
                if(count($emailEx)>1)
                {
                    for($x=1;$x<count($emailEx);$x++)
                    {
                        $tempEmail = trim($emailEx[$x]);
                        if($tempEmail !== false && $tempEmail != "" && !in_array($tempEmail, $usedEmails) )
                        {
                            array_push($usedEmails, $tempEmail);
                            if($cc!=""){$cc.=",";}
                            $cc.=$tempEmail;
                        }
                    }
                }
                
                
                if($cc!="")
                {
                    $headers .= 'Cc: ' . $cc . "\r\n";
                }
                
		$subject = (get_option('draftnotifyemailsubject')!==false && get_option('draftnotifyemailsubject')!="")? get_option('draftnotifyemailsubject') : "Draft on '" . get_bloginfo('name') . "'";
                
		mail($recipient, $subject, $message, $headers);
            
                if(!in_array($id, $emailedEx))
                {
                    update_option('draftnotifyemailedlist', "{$alreadyEmailedList}|{$id}" );
                }
            }
	}
	
}
add_action('save_post', 'dddn_process');








/*
*	Start of the settings page.
*/
add_action('admin_menu', 'draftnotify_admin_add_page');
function draftnotify_admin_add_page() {
	add_options_page('Draft Notify Settings', 'Draft Notify Menu', 'manage_options', 'draftnotify', 'draftnotify_options_page');
	
	add_action('admin_init', 'plugin_admin_init');
}

function plugin_admin_init(){
register_setting( 'draftnotify_options', 'draftnotifyemail' );
register_setting( 'draftnotify_options', 'draftnotifyemailfrom' );
register_setting( 'draftnotify_options', 'draftnotifyemailreply' );
register_setting( 'draftnotify_options', 'draftnotifyemailsubject' );
register_setting( 'draftnotify_options', 'draftnotifyuserlevel' );
register_setting( 'draftnotify_options', 'draftnotifyauthorlevel' );
register_setting( 'draftnotify_options', 'draftnotifyemailrevisions' );
register_setting( 'draftnotify_options', 'draftnotifyemailedlist' );
register_setting( 'draftnotify_options', 'draftnotifyemailpostlink' );
register_setting( 'draftnotify_options', 'draftnotifyemailauthorname' );
register_setting( 'draftnotify_options', 'draftnotifyemailauthoremail' );
register_setting( 'draftnotify_options', 'draftnotifyemaillinkspam' );
register_setting( 'draftnotify_options', 'draftnotifyposttypes1' );
register_setting( 'draftnotify_options', 'draftnotifyposttypes2' );
register_setting( 'draftnotify_options', 'draftnotifycustomheader' );
register_setting( 'draftnotify_options', 'draftnotifycustomfooter' );
for($x=0;$x<20;$x++)
{
register_setting( 'draftnotify_options', "draftnotifyadminemail{$x}" );
}
}


function draftnotify_options_page() {?>
<div>
<h2>Draft Notify Plugin Options</h2>
Use these options to control who and how you get notified about authors creating new drafts.
<form method="post" action="options.php">
<?php 

settings_fields('draftnotify_options');
do_settings_sections('draftnotify_options'); 

echo "<table width='100%'><tr>
<td width='20%' valign='top'><b>Email Address:</b></td>
<td width='20%' valign='top'><input id='draftnotifyemail' name='draftnotifyemail' size='40' type='text' value='".get_option('draftnotifyemail')."' />
<td width='60%' valign='top'>Please enter an email address to send to a specific person.  Separate multiples with a comma (,).  Multiple addresses are CC'd</td>
</td></tr>
<tr>
<td width='20%' valign='top'><b>From Address:</b></td>
<td width='20%' valign='top'><input id='draftnotifyemailfrom' name='draftnotifyemailfrom' size='40' type='text' value='".get_option('draftnotifyemailfrom')."' />
<td width='60%' valign='top'>Please enter an email address to include as the from header.</td>
</td></tr>
<tr>
<td width='20%' valign='top'><b>Reply-To Address:</b></td>
<td width='20%' valign='top'><input id='draftnotifyemailreply' name='draftnotifyemailreply' size='40' type='text' value='".get_option('draftnotifyemailreply')."' />
<td width='60%' valign='top'>Please enter an email address to include as the reply-to header.</td>
</td></tr>
<tr>
<td width='20%' valign='top'><b>Subject Of Email:</b></td>
<td width='20%' valign='top'><input id='draftnotifyemailsubject' name='draftnotifyemailsubject' size='40' type='text' value='".get_option('draftnotifyemailsubject')."' />
<td width='60%' valign='top'>Leave blank for the default which includes your blog name.</td>
</td></tr>
<tr>
<td valign='top'><b>Send to all Above Access Level:</b></td>
<td valign='top'>";

// create a dynmaic list based on the available roles.
echo "<select id='draftnotifyuserlevel' name='draftnotifyuserlevel' > ";
$savedSelectVal = get_option('draftnotifyuserlevel');	
$temp =(strlen($savedSelectVal) == 0 || $savedSelectVal < 0)?" selected='selected'":"";
echo "<option value='-1' {$temp}>Use the email Address</option>";

$tempArr = get_editable_roles(); 

foreach($tempArr as $role=>$roleArr)
{
	$tempLevel = getTempLevel($roleArr['capabilities']);
	$temp =($savedSelectVal==$tempLevel)?" selected='selected'":"";
	echo "<option value='{$tempLevel}' {$temp}>{$role}</option>";
}
	
echo "</select></td>
<td valign='top'>If you would like to send to all users above an access level, please select that level in the dropdown.  <br>If you leave both blank, no email will be sent.  If you fill in both, it will use the email address entered.</td>
</tr><tr>
<td valign='top'><b>Send for authors of this access level or below:</b></td>
<td valign='top'>";

// create a dynmaic list based on the available roles.
echo "<select id='draftnotifyauthorlevel' name='draftnotifyauthorlevel' > ";
$savedSelectVal = get_option('draftnotifyauthorlevel', 0);	

$tempArr = get_editable_roles(); 

foreach($tempArr as $role=>$roleArr)
{
	$tempLevel = getTempLevel($roleArr['capabilities']);
	$temp =($savedSelectVal==$tempLevel)?" selected='selected'":"";
	echo "<option value='{$tempLevel}' {$temp}>{$role}</option>";
}
	
echo "</select></td>
<td valign='top'></td>
</tr>
<tr>
<td valign='top'><b>Send to Admins:</b></td>
<td valign='top'>";

// create a dynmaic list based on the admin roles.

$tempArr = get_users(['role' => 'Administrator']); 

$tempCount = 0;
foreach($tempArr as $role=>$roleArr)
{
    $savedSelectVal = get_option("draftnotifyadminemail{$tempCount}");
    $checked = ($savedSelectVal !== false && $savedSelectVal != "")?" checked":"";
    echo "<input type='checkbox' id='draftnotifyadminemail{$tempCount}' name='draftnotifyadminemail{$tempCount}' value='{$roleArr->user_email}'{$checked}><label for='draftnotifyadminemail{$tempCount}'>{$roleArr->display_name} - {$roleArr->user_email}</label>";
    $tempCount++;
}
	
echo "</select></td>
<td valign='top'>Admin Accounts are listed here.  Select those accounts which should be emailed.  These will be added to the list of emails you entered above and ignored if you select a whole user group to email.</td>
</tr>";

// Post Type
echo "<tr>
<td valign='top'><b>Send Emails for These types of articles:</b></td>
<td valign='top'>";
$savedSelectVal = get_option('draftnotifyposttypes1');
$checked = ($savedSelectVal == "page")?" checked":"";
echo "<input type='checkbox' id='draftnotifyposttypes1' name='draftnotifyposttypes1' value='page'{$checked}><label for='draftnotifyposttypes1'>Pages</label>";
$savedSelectVal = get_option('draftnotifyposttypes2');
$checked = ($savedSelectVal == "post")?" checked":"";
echo "<input type='checkbox' id='draftnotifyposttypes2' name='draftnotifyposttypes2' value='post'{$checked}><label for='draftnotifyposttypes2'>Posts</label>";


echo "</td>
<td valign='top'>Choose the types of artiles you want emails for.</td>
</tr>";

// Revisions.
echo "
<tr>
<td valign='top'><b>Send Emails when revisions are made:</b></td>
<td valign='top'><select id='draftnotifyemailrevisions' name='draftnotifyemailrevisions' > ";
$savedSelectVal = get_option('draftnotifyemailrevisions');	
$temp =($savedSelectVal == 1)?" selected='selected'":"";
$temp2 =($savedSelectVal == 0)?" selected='selected'":"";
echo "<option value='1' {$temp}>Yes, Recieve emails about all revisions</option>
    <option value='0' {$temp2}>No, Only email about the first draft</option>
</select></td>
<td valign='top'>If you would like an email every time someones saves a revision to the draft, select Yes, otherwise, select NO and you'll only get an email the first time the draft is saved.</td>
</tr>";
    
$savedLinkSpam = get_option('draftnotifyemaillinkspam');

// Spam plugin not released yet.
echo "<input id='draftnotifyemaillinkspam' name='draftnotifyemaillinkspam' type='hidden' value='{$savedLinkSpam}'>";

//$temp =(function_exists("spc_check_post"))? "Which it is":"You Do not have this installed";
//
//$checked = ($savedLinkSpam)?" checked='checked' ":"";
//echo"<tr>
//<td valign='top'><b>Check for Spam First:</b></td>
//<td valign='top'><input id='draftnotifyemaillinkspam' name='draftnotifyemaillinkspam' type='checkbox' {$checked}> Check to Check For Spam First</td>
//<td valign='top'>If you have the Touch Of Tech Spam Post Checker installed, <b>{$temp}</b>, clicking this will check for spam before sending you an email.</td>
//</tr>";
    
      
  echo"<tr><td colspan='3'><h3>Email Format Options</h3></td></tr>";  
$savedEmailPostLinkVal = get_option('draftnotifyemailpostlink');
$savedEmailAuthorNameVal = get_option('draftnotifyemailauthorname');
$savedEmailAuthorEmailAddressVal = get_option('draftnotifyemailauthoremail');

$checked = ($savedEmailPostLinkVal)?" checked='checked' ":"";
echo"<tr>
<td valign='top'><b>Include Link to Posts in Email:</b></td>
<td valign='top'><input id='draftnotifyemailpostlink' name='draftnotifyemailpostlink' type='checkbox' {$checked}> Check to send</td>
<td valign='top'>&nbsp;</td>
</tr>";

$checked = ($savedEmailAuthorNameVal)?" checked='checked' ":"";
echo"<tr>
<td valign='top'><b>Include Author's Real Name in Email:</b></td>
<td valign='top'><input id='draftnotifyemailauthorname' name='draftnotifyemailauthorname' type='checkbox' {$checked}> Check to send</td>
<td valign='top'>&nbsp;</td>
</tr>";

$checked = ($savedEmailAuthorEmailAddressVal)?" checked='checked' ":"";
echo"<tr>
<td valign='top'><b>Include Author's Email Address in Email:</b></td>
<td valign='top'><input id='draftnotifyemailauthoremail' name='draftnotifyemailauthoremail' type='checkbox' {$checked}> Check to send</td>
<td valign='top'>&nbsp;</td>
</tr>";

   // Header
echo "<tr>
<td valign='top'><b>Custom Header text:</b></td>
<td valign='top'><textarea id='draftnotifycustomheader' name='draftnotifycustomheader' rows='3' cols='75'>".get_option('draftnotifycustomheader')."</textarea>
<td valign='top'>Text to be placed at the beginning of every email.</td>
</td></tr>";
    
  // Footer
echo "<tr>
<td valign='top'><b>Custom Footer text:</b></td>
<td valign='top'><textarea id='draftnotifycustomfooter' name='draftnotifycustomfooter' rows='3' cols='75'>".get_option('draftnotifycustomfooter')."</textarea>
<td valign='top'>Text to be placed at the end of every email.</td>
</td></tr>";
    

echo "</table>";

 submit_button(); 
?>
</form></div>

<img style='float:right' width='244' height='228' src='http://www.touchoftechnology.com/wp-content/uploads/2013/02/Logo.jpg'><br><br>This plugin is brought to you by Touch of Tech, Inc.  <a href='http://www.touchoftechnology.com' target='_blank'>www.TouchOfTechnology.com</a><br>
If you have any problems with it, please leave a comment <a href='http://www.touchoftechnology.com/draft-notify-wp-plugin-to-send-an-email-when-an-author-saves-a-draft/'>here</a>.

<?php


}

function getTempLevel($arr)
{
    $tempLevel = -4;
    for($x=0;$x<=10;$x++)
    {
        if(isset($arr["level_{$x}"]) && $arr["level_{$x}"] == 1)
        {
            $tempLevel = $x;
        }
    }
    return $tempLevel;
}

?>
